/*
 * Copyright (C) 2020 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include "ukws_intelwindow_box.h"

#include <QString>
#include <QDate>
#include <QSize>
#include <QSizePolicy>
#include <QEvent>
#include <QResizeEvent>

#include <QDebug>
#include <QStyleOption>
#include <QPainter>
#include <QPainterPath>
#include <QSettings>
#include <QDir>
#include <QApplication>

#include "ukws_common.h"
#include "ukws_helper.h"

using namespace UkwsHelperXHeader;

#include <QX11Info>

#define NO_ADD_FRAME

static const QSize UKWS_TITLE_SIZE = QSize(UKWS_TITLE_WIDTH, UKWS_TITLE_HEIGHT);
static const QSize UKWS_ICON_SIZE = QSize(UKWS_ICON_DEFAULT_WIDTH, UKWS_ICON_DEFAULT_HEIGHT);
static const QSize UKWS_THUMBNAIL_SIZE = QSize(UKWS_THUMBNAIL_DEFAULT_WIDTH, UKWS_THUMBNAIL_DEFAULT_HEIGHT);

UkwsIntelWindowBox::UkwsIntelWindowBox(QWidget *parent, bool hasCloseButton) : QWidget(parent)
{
    hasButton = hasCloseButton; // alt-tab与视图，分离按钮和上滑效果
    dragable = false;
    winLeftOffset = 0;
    winRightOffset = 0;
    winTopOffset = 0;
    winBottomOffset = 0;
    frameXid = 0;
    hasFrame = false;
    drag = nullptr;
    isSelected = false;
    titleAutoHide = false;

    titleLabel = new UkwsWindowExtraLabel();
    iconLabel = new UkwsWindowExtraLabel();
    thumbnailLabel = new UkwsWindowExtraLabel();

    titleSize = UKWS_TITLE_SIZE;
    iconSize = UKWS_ICON_SIZE;
    winboxSize = QSize(UKWS_WINDOWBOX_WIDTH, UKWS_WINDOWBOX_HEIGHT);
    dragIconSize = QSize(0, 0);

    iconLabel->setAlignment(Qt::AlignCenter);
    thumbnailLabel->setAlignment(Qt::AlignCenter);

    iconTransformationMode = Qt::SmoothTransformation;
    thumbnailTransformationMode = Qt::SmoothTransformation;

    mainLayout = new QVBoxLayout();
    topBarLayout = new QHBoxLayout();

    wnckWin = NULL;
    windowRect.setRect(0, 0, 0, 0);

    titleLabel->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);

    // 自动缩放图片
    //	titleLabel->setScaledContents(true);
    iconLabel->setScaledContents(false);
    thumbnailLabel->setScaledContents(false);

    // 设置控件缩放方式
    QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    sizePolicy.setRetainSizeWhenHidden(true);
    iconLabel->setSizePolicy(sizePolicy);
    sizePolicy.setHorizontalPolicy(QSizePolicy::Expanding);
    titleLabel->setSizePolicy(sizePolicy);
    sizePolicy.setVerticalPolicy(QSizePolicy::Expanding);

    // 设置控件最大尺寸
    iconLabel->resize(QSize(UKWS_ICON_DEFAULT_SIZE, UKWS_ICON_DEFAULT_SIZE));
    titleLabel->setFixedHeight(UKWS_TITLE_DEFAULT_HEIGHT);
    titleLabel->setMinimumWidth(1);
    thumbnailLabel->setMinimumSize(QSize(1, 1));

    titleLabel->setContentsMargins(0, 0, UKWS_WINDOWBOX_PADDING, 0);

    /*
     * 设置控件布局（简单布局，直接手写）
     *
     *                 +------+-------+---+   -╮
     * topBarLayout -> | icon | title | X |    |
     *                 +------+-------+---+    |
     *                 |                  |    | -> mainLayout
     *                 |  thumbnail       |    |
     *                 |                  |    |
     *                 +--------------+---+   -╯
     *
     */

    //mainLayout->setSpacing(UKWS_THUMBNAIL_SPACE);
    mainLayout->setAlignment(Qt::AlignTop | Qt::AlignHCenter);
    //topBarLayout->setSpacing(UKWS_ICON_SPACE);
    topBarLayout->setAlignment(Qt::AlignTop | Qt::AlignLeft);

    topBarLayout->addWidget(iconLabel);
    topBarLayout->addWidget(titleLabel);

    //自定义关闭按钮
    if (hasButton) {
        m_closeBtn = new UkwsCloseButton();
        m_closeBtn->setAlignment(Qt::AlignCenter);
        connect(m_closeBtn, &UkwsCloseButton::buttonClicked, this, [=](){buttonclicked(true);});
        topBarLayout->addWidget(m_closeBtn);
        m_closeBtn->installEventFilter(this);
    }

    mainLayout->addLayout(topBarLayout);
    mainLayout->addWidget(thumbnailLabel);
    this->setLayout(mainLayout);

    // 为缩略图控件注册监视对象
    thumbnailLabel->installEventFilter(this);
    this->installEventFilter(this);
    if (hasButton) {
        this->setAttribute(Qt::WA_AcceptTouchEvents);
    }

    // 设置缩略图控件的objectName
    titleLabel->setObjectName(UKWS_OBJ_WINBOX_WIN_NAME);
    thumbnailLabel->setObjectName(UKWS_OBJ_WINBOX_THUMBNAIL);
    this->setObjectName(UKWS_OBJ_WINBOX);

    // 对齐设置暂时不生效，使用下面的方式修正
//    mainLayout->setContentsMargins(8, 0, 8, 8);
    mainLayout->setMargin(8);
    setThumbnailNormal();

//    connect(this, &UkwsIntelWindowBox::clicked, this, &UkwsIntelWindowBox::activateWnckWindow);

    scaleTimes = 0;
    scaleTimer.setSingleShot(true);
    scaleTimer.setTimerType(Qt::CoarseTimer);
    scaleTimer.setInterval(UKWS_DRAG_SCALE_INTERVAL_MS);
    connect(&scaleTimer, &QTimer::timeout, this, &UkwsIntelWindowBox::scaleDragPixmap);

    this->setAttribute(Qt::WA_TranslucentBackground, true);
    titleLabel->setAttribute(Qt::WA_TranslucentBackground, true);
    iconLabel->setAttribute(Qt::WA_TranslucentBackground, true);
    thumbnailLabel->setAttribute(Qt::WA_TranslucentBackground, true);


    this->setWindowBoxUnselected();
}

void UkwsIntelWindowBox::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);
    QStyleOption styleOpt;
    styleOpt.init(this);
    QPainter painter(this);
    style()->drawPrimitive(QStyle::PE_Widget, &styleOpt, &painter, this);

    if (isSelected) {
        QSize size = this->size();

        // 获取圆角边框区域
        QRectF rect;
        QPainterPath path;

        rect = QRectF(UKWS_WINDOWBOX_BORDER, UKWS_WINDOWBOX_BORDER,
                      size.width() - UKWS_WINDOWBOX_BORDER * 2,
                      size.height() - UKWS_WINDOWBOX_BORDER * 2);
        path.addRoundedRect(rect, UKWS_THUMBNAIL_RADIUS, UKWS_THUMBNAIL_RADIUS);
        rect = QRectF(0, 0, size.width(), size.height());
        path.addRoundedRect(rect, UKWS_THUMBNAIL_RADIUS + UKWS_WINDOWBOX_BORDER,
                            UKWS_THUMBNAIL_RADIUS + UKWS_WINDOWBOX_BORDER);

        painter.setRenderHint(QPainter::Antialiasing, false);
        painter.setBrush(QBrush(QColor(255, 255, 255, 128)));
        painter.setPen(QPen(QColor(255, 255, 255, 0)));
        painter.drawPath(path);
    }

    QWidget::paintEvent(event);
}

void UkwsIntelWindowBox::setSubWidgetSizeByThnSize(int w, int h)
{
    // icon，宽高32，上下间距0，上下边框0，右边距5；
    // 缩略图控件，外边距8，外边框4，边框紧贴图片，宽占用：(8 + 4) * 2 = 24，高占用：8 + 4 = 12；
    // 高度调整值：8 + 32 + 16 = 56；
    // 宽度调整值：32 + 5 = 37
    // 标题宽度调整值：
    int fixW, fixH;
    fixW = (UKWS_ICON_DEFAULT_WIDTH + UKWS_WINDOWBOX_PADDING);
    titleLabel->setFixedSize(w - fixW, UKWS_ICON_DEFAULT_HEIGHT);
    this->updateTitleBySize();

    // 缩略图控件，外边框4，边框紧贴图片，宽、高占用：4 * 2 = 8
    fixW = UKWS_WINDOWBOX_BORDER * 2;
    fixH = UKWS_WINDOWBOX_BORDER * 2;
    thumbnailLabel->setFixedSize(w + fixW, h + fixH);
}

void UkwsIntelWindowBox::setWinboxSizeByHeight(int height)
{
    // winbox边框4，内边距4，宽、高占用：（4 + 4） * 2 = 16；
    // icon，高32，上下间距0，上下边框0，宽、高占用：32；
    // 缩略图控件，外边距8，外边框4，边框紧贴图片，宽占用：(8 + 4) * 2 = 24，高占用：8 + 4 = 12；
    // 高度调整值：16 + 32 + 12 = 60；
    // 宽度调整值：24
    int fixW, fixH, thnHeight;
    fixW = (UKWS_THUMBNAIL_MARGIN + UKWS_WINDOWBOX_BORDER) * 2;
    fixH = (UKWS_WINDOWBOX_BORDER + UKWS_WINDOWBOX_PADDING) * 2
            + UKWS_ICON_DEFAULT_HEIGHT
            + (UKWS_THUMBNAIL_MARGIN + UKWS_WINDOWBOX_BORDER);
    thnHeight = height - fixH;

    int w, h;
    float scale;
    float maxScale = 20.0 / 9;
    int maxWidth = int(thnHeight * maxScale);
    w = thumbnailLabel->originalQPixmap.size().width();
    h = thumbnailLabel->originalQPixmap.size().height();

    if (config->thumbnailFixedHeight == "true") {
        qDebug()<<"thumbnailFixedHeight";
        // 计算缩放因子
        scale = (float)thnHeight / h;
        h = thnHeight;
        // 计算缩略图大小
        if (w * scale < 184) {
            scale = (float)184 / w;
        }
        w = int(w * scale);

    } else {
        // 计算缩放因子
        qDebug()<<"thumbnailFixedHeight2";
        if (h > thnHeight)
            scale = (float)thnHeight / h;
        else
            scale = 1;
        // 限制最大宽度，调节缩放因子
        if (w * scale > maxWidth)
            scale = (float)maxWidth / w;
        // 计算缩略图大小
        w = int(w * scale);
        h = int(h * scale);
    }

    this->setSubWidgetSizeByThnSize(w, h);

    this->setFixedSize(w + fixW, height);
}

WnckWindow *UkwsIntelWindowBox::getWnckWindow()
{
    return wnckWin;
}
void UkwsIntelWindowBox::setWnckWindow(WnckWindow *window)
{
    int x, y, w, h;
    wnckWin = window;
    wnck_window_get_client_window_geometry(window, &x, &y, &w, &h);
    windowRect.setRect(x, y, w, h);
}

void UkwsIntelWindowBox::setTitle(QString title)
{
    this->title = title;
//    titleLabel->setText(this->title);
    this->updateTitleBySize();
}

void UkwsIntelWindowBox::updateTitleBySize()
{
    QFontMetrics fontMetrics(titleLabel->font());
    int fontSize = fontMetrics.width(this->title);
    QString formatTitle = this->title;
    if (fontSize > (titleLabel->width() - 5)) {
        formatTitle = fontMetrics.elidedText(this->title, Qt::ElideRight,
                                             titleLabel->width() - 15);
    }

    titleLabel->setText(formatTitle);
}

void UkwsIntelWindowBox::setIcon(QPixmap icon)
{
    iconLabel->originalQPixmap = icon;
    iconLabel->setPixmap(iconLabel->originalQPixmap
                         .scaled(iconSize.width() - 0, iconSize.height() - 0,
                                 Qt::KeepAspectRatio, iconTransformationMode));
}

void UkwsIntelWindowBox::setIconByWnck()
{
    iconLabel->originalQPixmap = UkwsHelper::gdkPixbufToQPixmap(wnck_window_get_icon(wnckWin));
    iconLabel->setPixmap(iconLabel->originalQPixmap
                         .scaled(iconSize.width() - 0, iconSize.height() - 0,
                                 Qt::KeepAspectRatio, thumbnailTransformationMode));
}

void UkwsIntelWindowBox::fixFrameWindowArea()
{
    // 判断窗口是否被窗口管理器添加了装饰窗口frame window
    int frameX, frameY, frameWidth, frameHeight;
    int origX, origY, origWidth, origHeight;

    wnck_window_get_geometry(wnckWin, &frameX, &frameY, &frameWidth, &frameHeight);
    wnck_window_get_client_window_geometry(wnckWin, &origX, &origY, &origWidth, &origHeight);

    winX = frameX;
    winY = frameY;
    winWidth = frameWidth;
    winHeight = frameHeight;

#ifdef NO_ADD_FRAME
    winLeftOffset = 0;
    winRightOffset = 0;
    winTopOffset = 0;
    winBottomOffset = 0;
    frameXid = wnck_window_get_xid(wnckWin);
    hasFrame = false;
#else
    if ((origWidth == frameWidth) && (origHeight == frameHeight)) {
        // 无窗口装饰区
        winLeftOffset = 0;
        winRightOffset = 0;
        winTopOffset = 0;
        winBottomOffset = 0;
        frameXid = wnck_window_get_xid(wnckWin);
        hasFrame = false;
    } else {
        // 有窗口装饰区，获取装饰区
        XWindowAttributes attr;
        Display *display = QX11Info::display();
        XID parentXid = UkwsHelper::getParentWindowId(wnck_window_get_xid(wnckWin));

        if (parentXid == ~(unsigned long)0) {
            // 获取父窗口ID失败，使用自身的WID作为frame窗口的ID，偏移量全为0
            winX = origX;
            winY = origY;
            winWidth = origWidth;
            winHeight = origHeight;
            winLeftOffset = 0;
            winRightOffset = 0;
            winTopOffset = 0;
            winBottomOffset = 0;
            frameXid = wnck_window_get_xid(wnckWin);
            hasFrame = false;

            return;
        } else {
            // frame窗口为本窗口的父窗口
            frameXid = parentXid;
        }

        // 获取frame窗口属性
        XGetWindowAttributes(display, frameXid, &attr);

        // 过滤窗口阴影
        winLeftOffset = frameX - attr.x;
        winRightOffset = attr.width - winLeftOffset - frameWidth;
        winTopOffset = frameY - attr.y;
        winBottomOffset = attr.height - winTopOffset - frameHeight;

        // 边界值修正
        if (winLeftOffset < 0) winLeftOffset = 0;
        if (winRightOffset < 0) winRightOffset = 0;
        if (winTopOffset < 0) winTopOffset = 0;
        if (winBottomOffset < 0) winBottomOffset = 0;

        hasFrame = true;
    }
#endif
}

void UkwsIntelWindowBox::setOrigThumbnailByWnck()
{
    fixFrameWindowArea();
    thumbnailLabel->originalQPixmap = UkwsHelper::getThumbnailByXid(frameXid,
                                                                    winLeftOffset,
                                                                    winRightOffset,
                                                                    winTopOffset,
                                                                    winBottomOffset);
}
/*
// winbox 透明度变化 包括标题和图标
void UkwsIntelWindowBox::setUndragedThumbnail(UkwsIntelWindowBox *wb)
{
    QPixmap tempPixmap = thnUnselectedPixmap;
    QPainter painter;
    painter.begin(&tempPixmap);
    painter.fillRect(tempPixmap.rect(), QColor(0, 0, 0, 115));
    painter.end();
    wb->thumbnailLabel->setPixmap(tempPixmap);
    QCoreApplication::processEvents(QEventLoop::AllEvents, 10);
}

// 恢复缩略图到原状 恢复包括标题和图标
void UkwsIntelWindowBox::recoveryAllWinbox(UkwsIntelWindowBox *wb)
{
    QPixmap tempPixmap = thnUnselectedPixmap;
    wb->thumbnailLabel->setPixmap(tempPixmap);
    QCoreApplication::processEvents(QEventLoop::AllEvents, 10);
}
*/

// winbox 透明度变化 包括标题和图标
void UkwsIntelWindowBox::setDragedThumbnail(UkwsIntelWindowBox *wb)
{
    QPixmap thumbnailPixmap = thnUnselectedPixmap;
    QPainter painterThumbnail;
    painterThumbnail.begin(&thumbnailPixmap);
    painterThumbnail.fillRect(thumbnailPixmap.rect(), QColor(0, 0, 0, 115));
    painterThumbnail.end();

    QPixmap iconPixmap = iconLabel->originalQPixmap;
    QPainter painterIcon;
    painterIcon.begin(&iconPixmap);
    painterIcon.fillRect(iconPixmap.rect(), QColor(0, 0, 0, 115));
    painterIcon.end();

    wb->thumbnailLabel->setPixmap(thumbnailPixmap);
    wb->titleLabel->setStyleSheet("color:rgb(0,0,0);");
    wb->iconLabel->setPixmap(iconPixmap);

    QCoreApplication::processEvents(QEventLoop::AllEvents, 10);
}

// 恢复缩略图到原状 恢复包括标题和图标
void UkwsIntelWindowBox::recoveryWinbox(UkwsIntelWindowBox *wb)
{
    QPixmap thumbnailPixmap = thnUnselectedPixmap;
    QPixmap iconPixmap = iconLabel->originalQPixmap;

    wb->thumbnailLabel->setPixmap(thumbnailPixmap);
    wb->titleLabel->setStyleSheet("color:rgb(255,255,255);");
    wb->iconLabel->setPixmap(iconPixmap);

    QCoreApplication::processEvents(QEventLoop::AllEvents, 30);
}

void UkwsIntelWindowBox::updateThumbnail()
{
    QSize labelSize = thumbnailLabel->size();

    thnUnselectedPixmap = QPixmap(labelSize.width(), labelSize.height());
    thnUnselectedPixmap.fill(Qt::transparent);
//    thnSelectedPixmap = QPixmap(labelSize.width(), labelSize.height());
//    thnSelectedPixmap.fill(Qt::transparent);

    // 自绘边框宽度为4px，纯缩略图大小为长宽需要减8
    QPixmap origThn, radiusThn;
    origThn = thumbnailLabel->originalQPixmap.scaled(labelSize.width() - UKWS_WINDOWBOX_BORDER * 2,
                                                     labelSize.height() - UKWS_WINDOWBOX_BORDER * 2,
                                                     Qt::KeepAspectRatio,
                                                     Qt::SmoothTransformation);
    int fixW = UKWS_WINDOWBOX_BORDER * 2;
    if (labelSize.width()-fixW >184) {
        origThn = thumbnailLabel->originalQPixmap.scaled(labelSize.width() - fixW,
                                                         labelSize.height() - fixW,
                                                         Qt::KeepAspectRatio,
                                                         Qt::SmoothTransformation);
    } else {
        origThn = thumbnailLabel->originalQPixmap.scaled(thumbnailLabel->originalQPixmap.size().width(),
                                                         labelSize.height() - fixW,
                                                         Qt::KeepAspectRatio,
                                                         Qt::SmoothTransformation);
    }

    QPainter painter;
    radiusThn = makeRadiusPixmap(origThn, UKWS_THUMBNAIL_RADIUS);
    painter.begin(&thnUnselectedPixmap);
    painter.drawPixmap((thnUnselectedPixmap.width()-radiusThn.width())/2,
                       (thnUnselectedPixmap.height()-radiusThn.height())/2,
                       radiusThn.width(), radiusThn.height(), radiusThn);
    painter.end();
    thnSelectedPixmap = thnUnselectedPixmap.copy();

    // 获取圆角边框区域
    QRectF rect;
    QPainterPath path;

    rect = QRectF(UKWS_WINDOWBOX_BORDER, UKWS_WINDOWBOX_BORDER,
                  labelSize.width() - UKWS_WINDOWBOX_BORDER * 2,
                  labelSize.height() - UKWS_WINDOWBOX_BORDER * 2);
    path.addRoundedRect(rect, UKWS_THUMBNAIL_RADIUS, UKWS_THUMBNAIL_RADIUS);
    rect = QRectF(0, 0, labelSize.width(), labelSize.height());
    path.addRoundedRect(rect, UKWS_THUMBNAIL_RADIUS + UKWS_WINDOWBOX_BORDER,
                        UKWS_THUMBNAIL_RADIUS + UKWS_WINDOWBOX_BORDER);

    painter.begin(&thnSelectedPixmap);
    painter.setRenderHint(QPainter::Antialiasing, false);
    painter.setBrush(QBrush(QColor(255, 255, 255, 128)));
    painter.setPen(QPen(QColor(255, 255, 255, 0)));
    painter.drawPath(path);
    painter.end();
}

void UkwsIntelWindowBox::setThumbnail(QPixmap origPixmap)
{

    thumbnailLabel->originalQPixmap = origPixmap;
    updateThumbnail();
    thumbnailLabel->setPixmap(thnUnselectedPixmap);
}

void UkwsIntelWindowBox::setThumbnailByWnck()
{
    fixFrameWindowArea();
    QSize labelSize = thumbnailLabel->size();
    if (labelSize.width() == 0 || labelSize.height() == 0)
        thumbnailLabel->originalQPixmap = UkwsHelper::getThumbnailByXid(frameXid,
                                                                        winLeftOffset,
                                                                        winRightOffset,
                                                                        winTopOffset,
                                                                        winBottomOffset);
    updateThumbnail();
    thumbnailLabel->setPixmap(thnUnselectedPixmap);
}

void UkwsIntelWindowBox::setDragIconSize(QSize size)
{
    dragIconSize = size;
}

QPixmap UkwsIntelWindowBox::windowPixmap()
{
    return thumbnailLabel->originalQPixmap;
}

QString UkwsIntelWindowBox::getTitle()
{
    return this->title;
}

void UkwsIntelWindowBox::setThumbnailHover()
{
    if (titleAutoHide) {
        iconLabel->show();
        titleLabel->show();
    }

    thumbnailLabel->setPixmap(thnSelectedPixmap);
}

void UkwsIntelWindowBox::setThumbnailNormal()
{
    if (titleAutoHide) {
        iconLabel->hide();
        titleLabel->hide();
    }

    thumbnailLabel->setPixmap(thnUnselectedPixmap);
}

void UkwsIntelWindowBox::setWindowBoxSelected()
{
    isSelected = true;
    update();
}

void UkwsIntelWindowBox::setWindowBoxUnselected()
{
    isSelected = false;
    update();
}

void UkwsIntelWindowBox::setTitleAutoHide(bool autoHide)
{
    titleAutoHide = autoHide;

    // 设置了自动隐藏则即刻隐藏标题栏
    if (titleAutoHide) {
        iconLabel->hide();
        titleLabel->hide();
    }
}

void UkwsIntelWindowBox::moveToWorkspace(int wsIndex)
{
    WnckScreen *screen = wnck_window_get_screen(wnckWin);
    WnckWorkspace *workspace = wnck_screen_get_workspace(screen, wsIndex);
    wnck_window_move_to_workspace(wnckWin, workspace);
}

// 构建圆角图片
QPixmap UkwsIntelWindowBox::makeRadiusPixmap(QPixmap orig, int radius)
{
    // 构造圆角裁剪区域
    QRectF rect = QRectF(0, 0, orig.width(), orig.height());
    QPainterPath path;
    path.addRoundedRect(rect, radius, radius);

    // 设置新全透明背景图片
    QPixmap dest(orig.size());
    dest.fill(Qt::transparent);

    // 裁剪圆角区域，将原图贴到裁剪好的区域，构建圆角图片
    QPainter painter(&dest);
    painter.setClipPath(path);
    painter.drawPixmap(orig.rect(), orig);

    return dest;
}

void UkwsIntelWindowBox::scaleDragPixmap()
{
    if (scaleTimes < 0) {
        return;
    }

    if (scaleTimes == 0) {
        return;
    }

    if (drag == nullptr) {
        return;
    }

    scaleTimes--;
    scaleTimer.start();

    // 设置缩放大小
    QSize size = thnUnselectedPixmap.size() - scaleUnitSize * (UKWS_DRAG_SCALE_TIMES - scaleTimes);
    QPixmap pixmap = thnUnselectedPixmap.scaled(size, Qt::KeepAspectRatio,
                                            Qt::FastTransformation);

    drag->setPixmap(pixmap);
    update();
//    drag->setHotSpot(mouseEvent->pos() - thumbnailOffset);
}

void UkwsIntelWindowBox::mouseLeaveProcess()
{
    QPoint leftTopPoint = iconLabel->mapToGlobal(iconLabel->rect().topLeft());

    int leftTopX = leftTopPoint.x() + UKWS_BORDER_THREE_PIXEL;
    int leftTopY = leftTopPoint.y() + UKWS_BORDER_THREE_PIXEL;

    QPoint rightBottomPoint = thumbnailLabel->mapToGlobal(thumbnailLabel->rect().bottomRight());
    int rightBottomX =  rightBottomPoint.x() - UKWS_BORDER_THREE_PIXEL;
    int rightBottomY = rightBottomPoint.y() - UKWS_BORDER_THREE_PIXEL;

    QPoint mousePoint = QCursor::pos();
    int mousePosX = mousePoint.x();
    int mousePosY = mousePoint.y();

    if (mousePosX <= leftTopX || mousePosX >= rightBottomX
       || mousePosY <= leftTopY || mousePosY >= rightBottomY) {
        if (hasButton) {
            m_closeBtn->setBtnState(BTN_STATE_NORMAL);
        }
    }

    return;
}

bool UkwsIntelWindowBox::thumbnailDragProcess(QEvent *event)
{
    // 缩略图置灰
    QPixmap tempPixmap = thnUnselectedPixmap;
    QPainter painter;
    painter.begin(&tempPixmap);
    painter.fillRect(thnUnselectedPixmap.rect(), QColor(0, 0, 0, 159));
    painter.end();
    thumbnailLabel->setPixmap(tempPixmap);

    // 拖拽处理
    drag = new QDrag(this);
    QMimeData *mimeData = new QMimeData;
    QByteArray byteData;
    QDataStream dataStream(&byteData, QIODevice::WriteOnly);

    dataStream << parentIndex << index << title;
    mimeData->setData("application/x-dnditemdata", byteData);
    drag->setMimeData(mimeData);
    drag->setPixmap(thnUnselectedPixmap.scaled(dragIconSize, Qt::KeepAspectRatio,
                                                  Qt::FastTransformation));

    // 事件发生时的坐标，减去thn边框的宽度，获得实际图片上的坐标
    QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
    QPoint hotSpot = (mouseEvent->pos() - thumbnailOffset) *
            (float)drag->pixmap().size().width() / (float)thnUnselectedPixmap.width();
    drag->setHotSpot(hotSpot);

    // 设置缩放大小
    scaleUnitSize = (thnUnselectedPixmap.size() - drag->pixmap().size()) / UKWS_DRAG_SCALE_TIMES;

    // 设置拖拽图片缩放动画的定时器
    scaleTimes = UKWS_DRAG_SCALE_TIMES;
    scaleTimer.start();

    if (drag->exec()) {
        thumbnailLabel->setPixmap(thnUnselectedPixmap);
    } else {
        thumbnailLabel->setPixmap(thnUnselectedPixmap);
    }

    drag->deleteLater();
    drag = nullptr;

    return true;
}

bool UkwsIntelWindowBox::thumbnailMoveProcess(QEvent *event)
{
    QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
    QPoint distance = mouseEvent->globalPos() - mouseStartPoint;

    distance.setX(0);

    //改变窗口的位置
    if ( qAbs(distance.y()) >= UKWS_SHOW_DISTANCE ){
        if ( distance.y() < 0 ) {
//            emit draged(this); // 上滑时 处理其他的窗口透明度
           // emit dragedOwn(this); // 上滑时 处理上滑的窗口透明度
            this->move(windowTopLeftPoint + distance);
        }
        if ( ! windowMoved ) {
            windowMoved = true;
        }

    } else {
        windowMoved = false;
    }

    return true;
}

bool UkwsIntelWindowBox::enterLabelEventProcess(QObject *watched)
{
    if (hasButton && watched == m_closeBtn){
        m_closeBtn->setBtnState(BTN_STATE_ENTER);
        return true;
    } else if (watched == thumbnailLabel) {
        leftBtnPressed = false;
        setThumbnailHover();
    }
    if (hasButton)
        m_closeBtn->setBtnState(BTN_STATE_HOVER);

    return true;
}

bool UkwsIntelWindowBox::leaveLabelEventProcess(QObject *watched)
{
    if (watched == thumbnailLabel) {
        leftBtnPressed = false;
        setThumbnailNormal();
    } else if (hasButton && watched == m_closeBtn) {
        m_closeBtn->setBtnState(BTN_STATE_HOVER);
    }

    mouseLeaveProcess();

    return true;
}

bool UkwsIntelWindowBox::leftBtnPressEventProcess(QObject *watched, QEvent *event)
{
    QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
    if (mouseEvent->buttons() & Qt::LeftButton){
        if (hasButton && watched == m_closeBtn) {
            m_closeBtn->setBtnState(BTN_STATE_PRESS);
            return true;
        } else {
            leftBtnPressed = true;
            windowMoved = false;
            windowTopLeftPoint = this->frameGeometry().topLeft();
            mouseStartPoint = mouseEvent->globalPos();
            raise();
            return true;
        }
    }

    return false;
}

bool UkwsIntelWindowBox::leftBtnReleaseEventProcess(QObject *watched, QEvent *event)
{
    Q_UNUSED(watched);
    QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
    if (leftBtnPressed && (mouseEvent->button() == Qt::LeftButton)) {
            leftBtnPressed = false;
            if (! windowMoved){
                emit clicked(this);
            } else {
                windowMoved = false;
                int vDist = mouseStartPoint.y() - mouseEvent->globalPos().y();

//                emit recoveryAll(this); // 其他的缩略图透明度恢复
              //  emit recovery(this); // 上滑的窗口的缩略图透明度恢复

                if (vDist > UKWS_MOVE_CLOSE_DISTANCE){
                    emit buttonclicked(true);
                } else {
                    this->move(windowTopLeftPoint);
                }
            }

            return true;
    }

    return false;
}

bool UkwsIntelWindowBox::moveWindowEventProcess(QObject *watched, QEvent *event)
{
    if (!leftBtnPressed)
        return false;

    if (dragable){
        if (watched == thumbnailLabel){
            return thumbnailDragProcess(event);
        }
    } else if (hasButton) {  // alt-tab去除上滑功能
        return thumbnailMoveProcess(event);
    }

    return false;
}

bool UkwsIntelWindowBox::touchBeginEventProcess(QObject *watched, QEvent *event)
{
    Q_UNUSED(watched);

    //触摸的时候会引起鼠标进入事件，所以此处要将按钮状态设置成正常
    m_closeBtn->setBtnState(BTN_STATE_NORMAL);
    QTouchEvent *touchEvent = static_cast<QTouchEvent *>(event);

    raise();
    windowTopLeftPoint = this->frameGeometry().topLeft();
    touchPressed = true;
    windowMoved = false;
    touchEvent->accept();

    return true;
}

bool UkwsIntelWindowBox::tabletBeginEventProcess(QObject *watched, QEvent *event)
{
    Q_UNUSED(watched);

    QTabletEvent *tabletEvent = static_cast<QTabletEvent *>(event);
    if (tabletEvent->buttons() & Qt::LeftButton){
        if (hasButton && watched == m_closeBtn) {
            m_closeBtn->setBtnState(BTN_STATE_PRESS);
            return true;
        } else {
            leftBtnPressed = true;
            windowMoved = false;
            windowTopLeftPoint = this->frameGeometry().topLeft();
            mouseStartPoint = tabletEvent->globalPos();
            tabletEvent->accept();
            raise();
            return true;
        }
    }
    return false;


}

bool UkwsIntelWindowBox::touchUpdateEventProcess(QObject *watched, QEvent *event)
{
    Q_UNUSED(watched);

    QTouchEvent *touchEvent = static_cast<QTouchEvent *>(event);
    QList<QTouchEvent::TouchPoint> touchPoints = touchEvent->touchPoints();

    if (TOUCH_ONE_POINT == touchPoints.count())
    {
        QPoint touchScreenNowPoint = touchPoints.at(0).screenPos().toPoint();
        QPoint touchScreenStartPoint = touchPoints.at(0).startScreenPos().toPoint();

        if(touchEvent->touchPointStates() & Qt::TouchPointPressed){
            qDebug("touch press point(%d %d)", touchScreenStartPoint.x(), touchScreenStartPoint.y());
        }
        else if(touchEvent->touchPointStates() & Qt::TouchPointMoved){
            QPoint distance;
            distance.setX(0);
            distance.setY(touchScreenNowPoint.y() - touchScreenStartPoint.y());
            if (distance.y() < 0){
                this->move(windowTopLeftPoint + distance);
            }


            // 手指触摸容易产生抖动，因此添加消抖动的功能
            if (! windowMoved)
            {
                if ( qAbs(touchScreenNowPoint.x() - touchScreenStartPoint.x()) > UKWS_SHOW_DISTANCE
                     || qAbs(distance.y()) > UKWS_SHOW_DISTANCE) {
                    windowMoved = true;
//                    emit draged(this); //上滑时 处理其他的窗口透明度
                 //   emit dragedOwn(this); // 上滑时 处理上滑的窗口透明度
                }
            }
        }
        else if(touchEvent->touchPointStates() & Qt::TouchPointReleased){
            qDebug("touch release point(%d %d)", touchScreenStartPoint.x(), touchScreenStartPoint.y());
        }

        return true;
    }

    return false;
}

bool UkwsIntelWindowBox::tabletUpdateEventProcess(QObject *watched, QEvent *event)
{

    if (!leftBtnPressed)
        return false;

    if (dragable){
        if (watched == thumbnailLabel){
            return thumbnailDragProcess(event);
        }
    } else if (hasButton) {  // alt-tab去除上滑功能
        QTabletEvent *tabletEvent = static_cast<QTabletEvent *>(event);
        QPoint distance = tabletEvent->globalPos() - mouseStartPoint;

        distance.setX(0);

        //改变窗口的位置
        if ( qAbs(distance.y()) >= UKWS_SHOW_DISTANCE ){
            if ( distance.y() < 0 ) {
    //            emit draged(this); // 上滑时 处理其他的窗口透明度
               // emit dragedOwn(this); // 上滑时 处理上滑的窗口透明度
                this->move(windowTopLeftPoint + distance);
            }
            if ( ! windowMoved ) {
                windowMoved = true;
            }

        } else {
            windowMoved = false;
        }

        return true;

    }

    return false;



}

bool UkwsIntelWindowBox::touchEndEventProcess(QObject *watched, QEvent *event)
{
    Q_UNUSED(watched);

    QTouchEvent *touchEvent = static_cast<QTouchEvent *>(event);
    QList<QTouchEvent::TouchPoint> touchPoints = touchEvent->touchPoints();
    QPoint touchScreenEndPoint = touchPoints.at(0).screenPos().toPoint();
    QPoint touchScreenStartPoint = touchPoints.at(0).startScreenPos().toPoint();

    touchPressed = false;
    if (!windowMoved) {
        emit clicked(this); //点击进入
    } else {
        windowMoved = false;
        int vDist = touchScreenStartPoint.y() - touchScreenEndPoint.y();

//        emit recoveryAll(this); // 其他的缩略图透明度恢复
        emit recovery(this); // 上滑的窗口的缩略图透明度恢复

        if (vDist > UKWS_MOVE_CLOSE_DISTANCE) {
            emit buttonclicked(true); //关闭
        } else {
           this->move(windowTopLeftPoint);
        }
    }

    touchEvent->accept();
    return true;
}

bool UkwsIntelWindowBox::tabletEndEventProcess(QObject *watched, QEvent *event)
{

    Q_UNUSED(watched);
    QTabletEvent *tabletEvent = static_cast<QTabletEvent *>(event);
    if (leftBtnPressed && (tabletEvent->button() == Qt::LeftButton)) {
            leftBtnPressed = false;
            if (! windowMoved){
                emit clicked(this);
            } else {
                windowMoved = false;
                int vDist = mouseStartPoint.y() - tabletEvent->globalPos().y();

//                emit recoveryAll(this); // 其他的缩略图透明度恢复
              //  emit recovery(this); // 上滑的窗口的缩略图透明度恢复

                if (vDist > UKWS_MOVE_CLOSE_DISTANCE){
                    emit buttonclicked(true);
                } else {
                    this->move(windowTopLeftPoint);
                }
            }
            tabletEvent->accept();

            return true;
    }

    return false;
}

bool UkwsIntelWindowBox::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type())
    {
        case QEvent::Enter:
            return enterLabelEventProcess(watched);
        case QEvent::Leave:
            return leaveLabelEventProcess(watched);
        case QEvent::MouseButtonPress:
            return leftBtnPressEventProcess(watched, event);
        case QEvent::MouseButtonRelease:
            return leftBtnReleaseEventProcess(watched, event);
        case QEvent::MouseMove:
            return moveWindowEventProcess(watched, event);
        case QEvent::TouchBegin:
            return touchBeginEventProcess(watched, event);
        case QEvent::TouchUpdate:
            return touchUpdateEventProcess(watched, event);
        case QEvent::TouchEnd:
            return touchEndEventProcess(watched, event);
       case QEvent::TabletPress:
            return tabletBeginEventProcess(watched, event);
       case QEvent::TabletRelease:
            return tabletEndEventProcess(watched, event);
       case QEvent::TabletMove:
            return tabletUpdateEventProcess(watched, event);


        default:
            return false;
    }

    return false;
}

void UkwsIntelWindowBox::activateWnckWindow()
{
    WnckWorkspace *workspace = wnck_window_get_workspace(wnckWin);
    unsigned long timestamp = QX11Info::getTimestamp();

    wnck_workspace_activate(workspace, timestamp);
    wnck_window_activate(wnckWin, timestamp);
}
bool UkwsIntelWindowBox::windowIsAlive()
{

        return WNCK_IS_WINDOW(wnckWin);

}

void UkwsIntelWindowBox::closeWnckWindow()
{
    unsigned long timestamp = QX11Info::getTimestamp();
    wnck_window_close(wnckWin, timestamp);
}
